@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
+ * Copyright (c) 2012 National Institute of Informatics
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +36,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: address.c,v 1.42 2003/08/20 05:12:58 motonori Exp $";
+static char *_id_ = "$Id: address.c,v 1.44 2012/06/07 07:51:35 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -58,12 +59,12 @@ char *name;
 		ha = (struct hostalias *)MALLOC(sizeof(struct hostalias));
 		if (ha == NULL)
 		{
-			log(LOG_NOTICE, "out of memory (addmyalias)");
+			logg(LOG_NOTICE, "out of memory (addmyalias)");
 			return -1;
 		}
 		/* bzero(ha, sizeof(struct hostalias)); */
 		if (cnf.debug & DEBUG_ADDRESS)
-		log(LOG_DEBUG, "adding alias: %s", name);
+		logg(LOG_DEBUG, "adding alias: %s", name);
 		len = strlen(name);
 		/* strip trailing dot */
 		while (len > 0 && name[len-1] == '.')
@@ -114,7 +115,7 @@ u_char *addr;
 			hostp = (struct host *)MALLOC(sizeof(struct host));
 			if (hostp == NULL)
 			{
-				log(LOG_NOTICE,
+				logg(LOG_NOTICE,
 					"out of memory (addinetaddress)");
 				return -1;
 			}
@@ -153,7 +154,7 @@ u_char *addr;
 	if (addr == NULL)	/* just link from mxp */
 	{
 		if (cnf.debug & DEBUG_ADDRESS)
-		log(LOG_DEBUG, "address info for %s linked", mxp->name);
+		logg(LOG_DEBUG, "address info for %s linked", mxp->name);
 		return 0;
 	}
 
@@ -170,13 +171,13 @@ u_char *addr;
 		{
 		  case AF_INET:
 			bcopy(addr, &sin, sizeof(sin));
-			log(LOG_INFO, "got new address %s for %s, state reset",
+			logg(LOG_INFO, "got new address %s for %s, state reset",
 				inet_ntoa(sin), mxp->name);
 			break;
 #if INET6
 		  case AF_INET6:
 			inet_ntop(AF_INET6, addr, buf, sizeof(buf));
-			log(LOG_INFO, "got new address %s for %s, state reset",
+			logg(LOG_INFO, "got new address %s for %s, state reset",
 				buf, mxp->name);
 
 			break;
@@ -193,19 +194,19 @@ u_char *addr;
 		{
 		  case AF_INET:
 			bcopy(addr, &sin, sizeof(sin));
-			log(LOG_DEBUG, "DNS: registering %s=%s",
+			logg(LOG_DEBUG, "DNS: registering %s=%s",
 				mxp->name, inet_ntoa(sin));
 			break;
 #if INET6
 		  case AF_INET6:
 			inet_ntop(AF_INET6, addr, buf, sizeof(buf));
-			log(LOG_DEBUG, "DNS: registering %s=%s",
+			logg(LOG_DEBUG, "DNS: registering %s=%s",
 				mxp->name, buf);
 
 			break;
 #endif
 		  default:
-			log(LOG_NOTICE,
+			logg(LOG_NOTICE,
 				"DNS: invalid inet_domain %d at addinetaddress",
 				inet_domain);
 		} 
@@ -226,7 +227,7 @@ u_char *addr;
 	iap = (struct inetaddr *)MALLOC(sizeof(struct inetaddr));
 	if (iap == NULL)
 	{
-		log(LOG_NOTICE, "out of memory (addinetaddress)");
+		logg(LOG_NOTICE, "out of memory (addinetaddress)");
 		return -1;
 	}
 	bzero(iap, sizeof(struct inetaddr));
@@ -235,7 +236,7 @@ u_char *addr;
 	iap->address = (char *)MALLOC(len);
 	if (iap->address == NULL)
 	{
-		log(LOG_NOTICE, "out of memory (addinetaddress)");
+		logg(LOG_NOTICE, "out of memory (addinetaddress)");
 		return -1;
 	}
 	bcopy(addr, iap->address, len);
@@ -462,12 +463,12 @@ char *addr, *domain, *notify, *orcpt;
 	struct domain *d, **hashp;
 
 	if (cnf.debug & DEBUG_LMTP)
-	log(LOG_DEBUG, "RCPT TO: %s notify=%s orcpt=%s", addr,
+	logg(LOG_DEBUG, "RCPT TO: %s notify=%s orcpt=%s", addr,
 		(notify == NULL)?"":notify, (orcpt == NULL)?"":orcpt);
 	rcpt = (struct recipient *)MALLOC(sizeof(struct recipient));
 	if (rcpt == NULL)
 	{
-		log(LOG_NOTICE, "out of memory (addrecipient)");
+		logg(LOG_NOTICE, "out of memory (addrecipient)");
 		return -1;
 	}
 	bzero(rcpt, sizeof(struct recipient));
@@ -493,7 +494,7 @@ char *addr, *domain, *notify, *orcpt;
 	if (d == NULL)
 	{
 		if (cnf.debug & DEBUG_LMTP)
-		log(LOG_DEBUG, "new domain: %s", domain);
+		logg(LOG_DEBUG, "new domain: %s", domain);
 		rcpt->domain = (struct domain *)MALLOC(sizeof(struct domain));
 		bzero(rcpt->domain, sizeof(struct domain));
 		rcpt->domain->name = newstr(domain);
@@ -546,7 +547,7 @@ struct recipient *rcpt;
 int rcode;
 char *response;
 {
-	log(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",            
+	logg(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",            
 		sti.nsent, sti.ndeferred, sti.nnsfailed,
 		sti.nsmtpfailed, sti.nrcpt, "unknown",
 		rcpt->address, "unknown", 0, rcode,   
