@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
+ * Copyright (c) 2012 National Institute of Informatics
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +36,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: dns.c,v 1.86 2003/08/20 05:12:58 motonori Exp $";
+static char *_id_ = "$Id: dns.c,v 1.90 2015/02/17 14:00:55 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -79,6 +80,12 @@ static char *_id_ = "$Id: dns.c,v 1.86 2003/08/20 05:12:58 motonori Exp $";
 # define RES_INSECURE1	0x00000400	/* type 1 security disabled */
 #endif
 
+static int query_domain(), process_query();
+static int make_query();
+static void got_answer();
+static void remove_query();
+static struct mx* newmx();
+
 int
 pregetmx(dl)
 struct domain *dl;
@@ -87,7 +94,6 @@ struct domain *dl;
 	char **map_arg;
 	long stime, etime;
 	int n;
-	static int query_domain(), process_query();
 #ifdef INET6
 	SockAddr dsin;	/* dummy */
 #endif
@@ -109,7 +115,7 @@ struct domain *dl;
 		}
 	}
 
-	log(LOG_DEBUG, "start getting MXRRs with caching");
+	logg(LOG_DEBUG, "start getting MXRRs with caching");
 	n = 0;
 	stime = time(NULL);
 	if (env.queueid != NULL)
@@ -120,7 +126,7 @@ struct domain *dl;
 	if (cnf.pgateway != NULL)
 	{
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "pregetting A of protocol gateway: %s",
+		logg(LOG_DEBUG, "pregetting A of protocol gateway: %s",
 			cnf.pgateway);
 #ifdef INET6
 		if (cnf.inetdom & SMTP_V4)
@@ -144,7 +150,7 @@ struct domain *dl;
 	if (cnf.fallbackmx != NULL)
 	{
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "pregetting A of fallbackmx: %s",
+		logg(LOG_DEBUG, "pregetting A of fallbackmx: %s",
 			cnf.fallbackmx);
 #ifdef INET6
 		if (cnf.inetdom & SMTP_V4)
@@ -179,13 +185,13 @@ struct domain *dl;
 
 #ifdef INET6
 			if (inet_addr(p) == -1
-			 && inet_pton(INET6, p, &dsin.in6.sin6_addr) <= 0)
+			 && inet_pton(AF_INET6, p, &dsin.in6.sin6_addr) <= 0)
 #else
 			if (inet_addr(p) == -1)
 #endif
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"pregetting A of sourceIP: %s", p);
 #ifdef INET6
 				if (cnf.inetdom & SMTP_V4)
@@ -243,7 +249,7 @@ struct domain *dl;
 			 || (strcasecmp(*map_arg, "MX?") == 0))
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "pregetting MX: %s", dp->name);
+				logg(LOG_DEBUG, "pregetting MX: %s", dp->name);
 				if (query_domain(dp->name, T_MX) < 0)
 					return -1;
 				n++;
@@ -251,7 +257,7 @@ struct domain *dl;
 			else if (strcasecmp(*map_arg, "A") == 0)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "pregetting A: %s", dp->name);
+				logg(LOG_DEBUG, "pregetting A: %s", dp->name);
 #ifdef INET6
 				if (cnf.inetdom & SMTP_V4)
 				{
@@ -272,7 +278,7 @@ struct domain *dl;
 			else if (**map_arg != '[')
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "pregetting MX: %s", *map_arg);
+				logg(LOG_DEBUG, "pregetting MX: %s", *map_arg);
 				if (query_domain(*map_arg, T_MX) < 0)
 					return -1;
 				n++;
@@ -283,7 +289,7 @@ struct domain *dl;
 				(*map_arg)[len - 1] = '\0';
 #ifdef INET6
 				if (inet_addr((*map_arg) + 1) != -1
-				 || inet_pton(INET6, (*map_arg) + 1,
+				 || inet_pton(AF_INET6, (*map_arg) + 1,
 					      &dsin.in6.sin6_addr) > 0)
 #else
 				if (inet_addr((*map_arg) + 1) != -1)
@@ -291,7 +297,7 @@ struct domain *dl;
 					goto skip_preget_a;
 
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"pregetting A of sourceIP: %s",
 					(*map_arg) + 1);
 #ifdef INET6
@@ -333,16 +339,16 @@ struct domain *dl;
 		else
 			setproctitle("syncing MX: %d", n);
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "syncing responses of MXRRs");
+		logg(LOG_DEBUG, "syncing responses of MXRRs");
 		if (process_query(1) < 0)	/* sync */
 			return -1;
 		etime = time(NULL);
-		log(LOG_DEBUG, "got MXRRs: %d MXs in %d seconds",
+		logg(LOG_DEBUG, "got MXRRs: %d MXs in %d seconds",
 			n, etime - stime);
 	}
 	else
 	{
-		log(LOG_DEBUG, "no MXRRs to sync");
+		logg(LOG_DEBUG, "no MXRRs to sync");
 	}
 	return 0;
 }
@@ -353,7 +359,6 @@ char *name;
 int type;
 {
 	struct dns_stat *qrp, **hashp;
-	static int make_query();
 
 	qrp = hash_query_lookup(name, &hashp);
 	if (qrp == NULL) {
@@ -361,7 +366,7 @@ int type;
 		bzero(qrp, sizeof(struct dns_stat));
 		if (qrp == NULL)
 		{
-			log(LOG_NOTICE, "out of memory (dns_query)");
+			logg(LOG_NOTICE, "out of memory (dns_query)");
 			return -1;
 		}
 		qrp->name = newstr(name);
@@ -375,7 +380,7 @@ int type;
 	  case T_AAAA:
 		break;
 	  default:
-		log(LOG_NOTICE, "query_domain: unknown query type: %d", type);
+		logg(LOG_NOTICE, "query_domain: unknown query type: %d", type);
 		return 0;
 	}
 
@@ -396,7 +401,6 @@ int type;
 	u_char buf[MAXPACKET];
 	int n;
 	struct query_chain *qchain;
-	static int process_query();
 
 	switch (type) {
 	  case T_MX:
@@ -419,7 +423,7 @@ int type;
 	{
 		/* already got */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "ignore request n=%s t=%d",
+		logg(LOG_DEBUG, "ignore request n=%s t=%d",
 			qrp->name, type);
 		return 0;
 	}
@@ -439,18 +443,18 @@ int type;
 	{
 		/* already in processing */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "skip request n=%s t=%d", qrp->name, type);
+		logg(LOG_DEBUG, "skip request n=%s t=%d", qrp->name, type);
 		return 0;
 	}
 
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "res_mkquery n=%s t=%d", qrp->name, type);
+	logg(LOG_DEBUG, "res_mkquery n=%s t=%d", qrp->name, type);
 	n = res_mkquery(QUERY, qrp->name, C_IN, type, NULL, 0, NULL,
 			buf, sizeof(buf));
 	if (n <= 0)
 	{
 		/* error */
-		log(LOG_NOTICE, "res_mkquery failed for %s type %d",
+		logg(LOG_NOTICE, "res_mkquery failed for %s type %d",
 			qrp->name, type);
 		return -1;
 	}
@@ -459,7 +463,7 @@ int type;
 	if (qchain == NULL)
 	{
 		/* error */
-		log(LOG_NOTICE, "out of memory (make_query)");
+		logg(LOG_NOTICE, "out of memory (make_query)");
 		return -1;
 	}
 	bzero(qchain, sizeof(struct query_chain));
@@ -469,7 +473,7 @@ int type;
 	if (qchain->request == NULL)
 	{
 		/* error */
-		log(LOG_NOTICE, "out of memory (make_query)");
+		logg(LOG_NOTICE, "out of memory (make_query)");
 		return -1;
 	}
 	bcopy(buf, qchain->request, n);
@@ -492,7 +496,7 @@ int type;
 	qchain->prev = NULL;
 	query_chain_count++;
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "DNS: query concurrency = %d", query_chain_count);
+	logg(LOG_DEBUG, "DNS: query concurrency = %d", query_chain_count);
 	if (sti.maxcquery < query_chain_count)
 		sti.maxcquery = query_chain_count;
 
@@ -501,10 +505,10 @@ int type;
 	{
 		struct query_chain *qcp;
 
-		log(LOG_DEBUG, "Chain after linked:");
+		logg(LOG_DEBUG, "Chain after linked:");
 		for (qcp = QueryChain; qcp != NULL; qcp = qcp->next)
 		{
-			log(LOG_DEBUG, " %s", qcp->stat->name);
+			logg(LOG_DEBUG, " %s", qcp->stat->name);
 		}
 	}
 #endif
@@ -591,14 +595,12 @@ int sync;
 	char nbuf[MAXDNAME+1];
 	struct dns_stat *qrp;
 	int pending;
-	static void got_answer();
-	static void remove_query();
 
 	if (sync == 0 && cnf.cquery_max != 0
 	 && query_chain_count >= cnf.cquery_max)
 	{
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "reached to max concurrency (%d)....syncing",
+		logg(LOG_DEBUG, "reached to max concurrency (%d)....syncing",
 			query_chain_count);
 		sync = -1;
 	}
@@ -612,7 +614,7 @@ int sync;
 			s = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
 			if (s < 0) {
 				/* error */
-				log(LOG_NOTICE,
+				logg(LOG_NOTICE,
 					"socket failed (process_query)");
 				return -1;
 			}
@@ -622,8 +624,8 @@ int sync;
 			s6 = socket(AF_INET6, SOCK_DGRAM, PF_UNSPEC);
 			if (s6 < 0) {
 				/* error */
-				log(LOG_NOTICE,
-					"socket(INET6) failed (process_query)");
+				logg(LOG_NOTICE,
+					"socket(AF_INET6) failed (process_query)");
 				return -1;
 			}
 		}
@@ -633,7 +635,7 @@ int sync;
 		s = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
 		if (s < 0) {
 			/* error */
-			log(LOG_NOTICE, "socket failed (process_query)");
+			logg(LOG_NOTICE, "socket failed (process_query)");
 			return -1;
 		}
 	}
@@ -663,14 +665,14 @@ again:
 					nextqcp = qcp->next;
 					remove_query(qcp);
 					if (cnf.debug & DEBUG_DNS)
-					log(LOG_DEBUG, "DNS: no valid ns, removing %s type %d",
+					logg(LOG_DEBUG, "DNS: no valid ns, removing %s type %d",
 						qcp->stat->name, qcp->type);
 					qcp = nextqcp;
 					continue;
 				}
 				qcp->try++;
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: retry count=%d", qcp->try);
+				logg(LOG_DEBUG, "DNS: retry count=%d", qcp->try);
 			}
 		}
 		if (qcp->try >= (u_char)_res.retry)
@@ -680,7 +682,7 @@ again:
 			nextqcp = qcp->next;
 			remove_query(qcp);
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: no valid ns, removing %s type %d",
+			logg(LOG_DEBUG, "DNS: no valid ns, removing %s type %d",
 				qcp->stat->name, qcp->type);
 			qcp = nextqcp;
 			continue;
@@ -689,7 +691,7 @@ again:
 		{
 			itvl = _res.retrans << qcp->try;
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: waiting %d sec.", itvl);
+			logg(LOG_DEBUG, "DNS: waiting %d sec.", itvl);
 			if (qcp->try > 0)
 				itvl /= _res.nscount;
 			if (itvl <= 0)
@@ -700,7 +702,7 @@ again:
 			if (nsap->in.sin_family == AF_INET)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
+				logg(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
 					qcp->stat->name, qcp->type,
 					inet_ntoa(nsap->in.sin_addr));
 
@@ -711,7 +713,7 @@ again:
 				{
 					qcp->badns |= 1<<qcp->ns;
 					if (cnf.debug & DEBUG_DNS)
-					log(LOG_DEBUG, "sendto failed: invalid ns (%s)", strerror(errno));
+					logg(LOG_DEBUG, "sendto failed: invalid ns (%s)", strerror(errno));
 				}
 			}
 			if (nsap->in6.sin6_family == AF_INET6)
@@ -722,7 +724,7 @@ again:
 				{
 				inet_ntop(AF_INET6, &nsap->in6.sin6_addr,
 					buf, sizeof(buf));
-				log(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
+				logg(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
 					qcp->stat->name, qcp->type, buf);
 				}
 
@@ -733,13 +735,13 @@ again:
 				{
 					qcp->badns |= 1<<qcp->ns;
 					if (cnf.debug & DEBUG_DNS)
-					log(LOG_DEBUG, "sendto failed: invalid ns (%s)", strerror(errno));
+					logg(LOG_DEBUG, "sendto failed: invalid ns (%s)", strerror(errno));
 				}
 			}
 #else
 			nsap = &NSADDR_LIST(qcp->ns);
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
+			logg(LOG_DEBUG, "send query n=%s t=%d (s=%s)",
 				qcp->stat->name, qcp->type,
 				inet_ntoa(nsap->sin_addr));
 
@@ -749,7 +751,7 @@ again:
 			{
 				qcp->badns |= 1<<qcp->ns;
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "sendto failed: invalid ns");
+				logg(LOG_DEBUG, "sendto failed: invalid ns");
 			}
 #endif
 			sti.nqueries++;
@@ -793,7 +795,7 @@ again:
 			timeout.tv_usec = 0;
 		}
 		if ((cnf.debug & DEBUG_DNS) && (timeout.tv_sec > 0))
-		log(LOG_DEBUG, "DNS: waiting %d sec in %s mode",
+		logg(LOG_DEBUG, "DNS: waiting %d sec in %s mode",
 			timeout.tv_sec, sync?"sync":"nosync");
 		FD_ZERO(&dsmask);
 #if INET6
@@ -825,7 +827,7 @@ again:
 				resplen = recvfrom(s, &answer, anssiz, 0,
 					   (struct sockaddr *)&from, &fromlen);
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got answer from %s",
+				logg(LOG_DEBUG, "DNS: got answer from %s",
 					inet_ntoa(from.in.sin_addr));
 			}
 			else if (s6 >= 0 && FD_ISSET(s6, &dsmask))
@@ -838,7 +840,7 @@ again:
 				{
 					inet_ntop(AF_INET6, &from.in6.sin6_addr,
 						buf, sizeof(buf));
-					log(LOG_DEBUG,
+					logg(LOG_DEBUG,
 						"DNS: got answer from %s", buf);
 				}
 			}
@@ -846,7 +848,7 @@ again:
 			resplen = recvfrom(s, &answer, anssiz, 0,
 					   (struct sockaddr *)&from, &fromlen);
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: got answer from %s",
+			logg(LOG_DEBUG, "DNS: got answer from %s",
 				inet_ntoa(from.sin_addr));
 #endif
 
@@ -858,7 +860,7 @@ again:
 
 			if (resplen <= 0) {
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: bad anssiz %d",
+				logg(LOG_DEBUG, "DNS: bad anssiz %d",
 					resplen);
 				continue;	/* error */
 			}
@@ -870,7 +872,7 @@ again:
 				if (!(_res.options & RES_INSECURE1))
 				{
 					if (cnf.debug & DEBUG_DNS)
-					log(LOG_DEBUG,
+					logg(LOG_DEBUG,
 						"DNS: not from our server");
 					continue;
 				}
@@ -888,7 +890,7 @@ again:
 			GETSHORT(class, ap);
  
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: got answer for %s type %d",
+			logg(LOG_DEBUG, "DNS: got answer for %s type %d",
 				nbuf, type);
 
 			qrp = hash_query_lookup(nbuf, NULL);
@@ -915,7 +917,7 @@ again:
 			{
 				qcp->badns |= (1<<ns);
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: badnsF rcode=%d %x/%x",
+				logg(LOG_DEBUG, "DNS: badnsF rcode=%d %x/%x",
 					hp->rcode,
 					qcp->badns, (1<<_res.nscount)-1);
 				if (qcp->badns != (1<<_res.nscount)-1)
@@ -924,7 +926,7 @@ again:
 				qrp->rr_timeout |= qcp->need;
 				remove_query(qcp);
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: TCP requested, removing %s type %d",
+				logg(LOG_DEBUG, "DNS: TCP requested, removing %s type %d",
 					qcp->stat->name, qcp->type);
 				continue;
 			}
@@ -933,7 +935,7 @@ again:
 				qrp->rr_timeout |= qcp->need;
 				remove_query(qcp);
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: TCP requested, removing %s type %d",
+				logg(LOG_DEBUG, "DNS: TCP requested, removing %s type %d",
 					qcp->stat->name, qcp->type);
 				continue;
 			}
@@ -952,7 +954,7 @@ again:
 	if (sync != 0) {
 		pending = 0;
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "DNS: sync: get nearest timeout from %d", now);
+		logg(LOG_DEBUG, "DNS: sync: get nearest timeout from %d", now);
 		for (qcp = QueryChain; qcp != NULL;)
 		{
 			if (qcp->try >= (u_char)_res.retry)
@@ -962,7 +964,7 @@ again:
 				nextqcp = qcp->next;
 				remove_query(qcp);
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: sync timeout (n=%s, t=%d)",
+				logg(LOG_DEBUG, "DNS: sync timeout (n=%s, t=%d)",
 					qcp->stat->name, qcp->type);
 				qcp = nextqcp;
 				continue;
@@ -970,7 +972,7 @@ again:
 
 			pending++;
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: event timeout=%d", qcp->time);
+			logg(LOG_DEBUG, "DNS: event timeout=%d", qcp->time);
 			qcp = qcp->next;
 		}
 		if (sync < 0)
@@ -978,7 +980,7 @@ again:
 		if (pending)
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: retry in sync mode");
+			logg(LOG_DEBUG, "DNS: retry in sync mode");
 			/* sleep(1); * XXX */
 			goto again;
 		}
@@ -989,12 +991,12 @@ again:
 			nextqcp = qcp->next;
 			remove_query(qcp);
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: final timeout (n=%s, t=%d)",
+			logg(LOG_DEBUG, "DNS: final timeout (n=%s, t=%d)",
 				qcp->stat->name, qcp->type);
 			qcp = nextqcp;
 		}
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "DNS: closing socket");
+		logg(LOG_DEBUG, "DNS: closing socket");
 #if INET6
 		if (s >= 0)
 		{
@@ -1038,10 +1040,10 @@ struct query_chain *qcp;
 #if 0
 	if (cnf.debug & DEBUG_DNS)
 	{
-		log(LOG_DEBUG, "Chain after removed:");
+		logg(LOG_DEBUG, "Chain after removed:");
 		for (qcp = QueryChain; qcp != NULL; qcp = qcp->next)
 		{
-			log(LOG_DEBUG, " %s", qcp->stat->name);
+			logg(LOG_DEBUG, " %s", qcp->stat->name);
 		}
 	}
 #endif
@@ -1069,7 +1071,7 @@ int anssiz;
 #endif
 
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "DNS: got answer, removing %s type %d",
+	logg(LOG_DEBUG, "DNS: got answer, removing %s type %d",
 		qcp->stat->name, qcp->type);
 
 	b  = (u_char *) answer;
@@ -1080,7 +1082,7 @@ int anssiz;
 	if (hp->aa == 0 && hp->ra == 0) {
 		/* this nameserver does not support recursion */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "non recursive ns=%d", qcp->ns);
+		logg(LOG_DEBUG, "non recursive ns=%d", qcp->ns);
 		qcp->badns |= 1<<qcp->ns;
 		return;	/* keep waiting for a reply from other NS */
 	}
@@ -1098,7 +1100,7 @@ int anssiz;
 	{
 		/* host not found */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "DNS: host (any other data) not found");
+		logg(LOG_DEBUG, "DNS: host (any other data) not found");
 
 		qcp->stat->rr_noex = RR_ALL;
 		return;
@@ -1109,7 +1111,7 @@ int anssiz;
 	{
 		/* no data, other data exist */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "DNS: no data for %d", qcp->type);
+		logg(LOG_DEBUG, "DNS: no data for %d", qcp->type);
 
 		if (qcp->type == T_ANY)
 		{
@@ -1153,7 +1155,7 @@ int anssiz;
 	}
 
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "DNS: valid data");
+	logg(LOG_DEBUG, "DNS: valid data");
 
 	while (qdcount--)
 	{
@@ -1161,7 +1163,7 @@ int anssiz;
 		ap += n;
 		if (ap + INT16SZ*2 > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(type, ap);
@@ -1181,7 +1183,7 @@ int anssiz;
 		ap += n;
 		if (ap + INT16SZ*3 + INT32SZ > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(type, ap);
@@ -1190,13 +1192,13 @@ int anssiz;
 		GETSHORT(n, ap);
 		if (ap + n > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		if (class != C_IN)
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: unexpected class %d in an", class);
+			logg(LOG_DEBUG, "DNS: unexpected class %d in an", class);
 			ap += n;
 			continue;
 		}
@@ -1230,7 +1232,7 @@ int anssiz;
 					return;
 
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got MX for %s, %s p=%d",
+				logg(LOG_DEBUG, "DNS: got MX for %s, %s p=%d",
 					nbuf, nbuf2, pref);
 
 				qrp->rr_got |= RR_MX;
@@ -1269,7 +1271,7 @@ int anssiz;
 			{
 				addr_found = 1;
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got AAAA for %s",
+				logg(LOG_DEBUG, "DNS: got AAAA for %s",
 					qcp->stat->name);
 
 				qcp->stat->rr_got |= RR_AAAA;
@@ -1279,7 +1281,7 @@ int anssiz;
 				/* may be address for an alias */
 				cname_found = 0;
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got aliased (%s) AAAA for %s",
+				logg(LOG_DEBUG, "DNS: got aliased (%s) AAAA for %s",
 					nbuf, qcp->stat->name);
 				qrp = hash_query_lookup(nbuf, NULL);
 				if (qrp == NULL)
@@ -1310,7 +1312,7 @@ int anssiz;
 			{
 				addr_found = 1;
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got A for %s",
+				logg(LOG_DEBUG, "DNS: got A for %s",
 					qcp->stat->name);
 
 				qcp->stat->rr_got |= RR_A;
@@ -1320,7 +1322,7 @@ int anssiz;
 				cname_found = 0;
 				/* may be address for an alias */
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: got aliased (%s) A for %s",
+				logg(LOG_DEBUG, "DNS: got aliased (%s) A for %s",
 					nbuf, qcp->stat->name);
 				qrp = hash_query_lookup(nbuf, NULL);
 				if (qrp == NULL)
@@ -1343,13 +1345,13 @@ int anssiz;
 				return;
 
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: got CNAME %s for %s",
+			logg(LOG_DEBUG, "DNS: got CNAME %s for %s",
 				nbuf2, nbuf);
 
 			if (strcasecmp(nbuf, nbuf2) == 0)
 			{
 				/* CNAME points back itself, ignore it */
-				log(LOG_INFO,
+				logg(LOG_INFO,
 					"CNAME %s points back itself (q=%d)",
 					nbuf, qcp->type);
 				return;
@@ -1377,7 +1379,7 @@ int anssiz;
 				bzero(qrp2, sizeof(struct dns_stat));
 				if (qrp2 == NULL)
 				{
-					log(LOG_NOTICE,
+					logg(LOG_NOTICE,
 						"out of memory (dns_query)");
 					return;
 				}
@@ -1442,13 +1444,13 @@ int anssiz;
 		ap += INT16SZ + INT16SZ + INT32SZ;
 		if (ap + INT16SZ > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(n, ap);
 		if (ap + n > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		ap += n;
@@ -1461,13 +1463,13 @@ int anssiz;
  		ap += INT16SZ + INT16SZ + INT32SZ;
 		if (ap + INT16SZ > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(n, ap);
 		if (ap + n > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		ap += n;
@@ -1482,7 +1484,7 @@ int anssiz;
 		ap += n;
 		if (ap + INT16SZ*3 + INT32SZ > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(type, ap);
@@ -1491,13 +1493,13 @@ int anssiz;
 		GETSHORT(addrsize, ap);
 		if (ap + addrsize > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		if (class != C_IN)
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: unexpected class %d in ar", class);
+			logg(LOG_DEBUG, "DNS: unexpected class %d in ar", class);
 			ap += addrsize;
 			continue;
 		}
@@ -1530,7 +1532,7 @@ int anssiz;
 		}
 
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "DNS: got additional Address RR for %s type=%d",
+		logg(LOG_DEBUG, "DNS: got additional Address RR for %s type=%d",
 			nbuf, type);
 
 #if FAKE_AR_CHECK
@@ -1546,7 +1548,7 @@ int anssiz;
 		if (!found)
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: ignoring additional Address RR");
+			logg(LOG_DEBUG, "DNS: ignoring additional Address RR");
 			ap += addrsize;
 			continue;
 		}
@@ -1559,7 +1561,7 @@ int anssiz;
 			bzero(qrp, sizeof(struct dns_stat));
 			if (qrp == NULL)
 			{
-				log(LOG_NOTICE, "out of memory (dns_query)");
+				logg(LOG_NOTICE, "out of memory (dns_query)");
 				return;
 			}
 			qrp->name = newstr(nbuf);
@@ -1593,7 +1595,7 @@ int anssiz;
 				bzero(qrp, sizeof(struct dns_stat));
 				if (qrp == NULL)
 				{
-					log(LOG_NOTICE,
+					logg(LOG_NOTICE,
 						"out of memory (dns_query)");
 					return;
 				}
@@ -1606,20 +1608,20 @@ int anssiz;
 			if (cnf.inetdom & SMTP_V6 || cnf.v4v6fallback)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: getting AAAA RR for %s",
+				logg(LOG_DEBUG, "DNS: getting AAAA RR for %s",
 					mxp->name);
 				make_query(qrp, T_AAAA);
 			}
 			if (cnf.inetdom & SMTP_V4 || cnf.v4v6fallback)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "DNS: getting A RR for %s",
+				logg(LOG_DEBUG, "DNS: getting A RR for %s",
 					mxp->name);
 				make_query(qrp, T_A);
 			}
 #else
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: getting A RR for %s",
+			logg(LOG_DEBUG, "DNS: getting A RR for %s",
 				mxp->name);
 			make_query(qrp, T_A);
 			if (cnf.v4v6fallback)
@@ -1654,7 +1656,7 @@ struct domain *dl;
 			continue;
 
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "getting MX for %s", dp->name);
+		logg(LOG_DEBUG, "getting MX for %s", dp->name);
 
 		offset = 0;
 		needfindaddr = 0;
@@ -1702,15 +1704,15 @@ struct domain *dl;
 		{
 			struct mx *mxp;
 
-			log(LOG_DEBUG, "final MX list for %s", dp->name);
+			logg(LOG_DEBUG, "final MX list for %s", dp->name);
 			mxp = dp->firstmx;
 			if (mxp == NULL)
 			{
-				log(LOG_DEBUG, " NO MX!");
+				logg(LOG_DEBUG, " NO MX!");
 			}
 			while (mxp != NULL)
 			{
-				log(LOG_DEBUG, " %s(%d, %d)", mxp->name,
+				logg(LOG_DEBUG, " %s(%d, %d)", mxp->name,
 					mxp->pref, mxp->weight);
 				mxp = mxp->next;
 			}
@@ -1734,12 +1736,12 @@ struct domain *dl;
 			int found = 0;
 
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "protocol gateway checking for %s",
+			logg(LOG_DEBUG, "protocol gateway checking for %s",
 				dp->name);
 			for (mxp = dp->firstmx; mxp != NULL; mxp = mxp->next)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "checking MX: %s", mxp->name);
+				logg(LOG_DEBUG, "checking MX: %s", mxp->name);
 				if (mxp->host == NULL)
 				{
 					/* no hosts, try next MX */
@@ -1751,7 +1753,7 @@ struct domain *dl;
 				{
 /*
 					if (cnf.debug & DEBUG_DNS)
-					log(LOG_DEBUG, "proto: %d",
+					logg(LOG_DEBUG, "proto: %d",
 						addrp->domain);
 */
 #ifdef INET6
@@ -1764,10 +1766,9 @@ struct domain *dl;
 #endif
 					{
 						struct mx *gwmxp;
-						static struct mx* newmx();
 
 						if (cnf.debug & DEBUG_DNS)
-						log(LOG_DEBUG, "setting protocol gateway as first MX");
+						logg(LOG_DEBUG, "setting protocol gateway as first MX");
 						if ((gwmxp = newmx(cnf.pgateway,
 								  -2, 0, dp, 0))
 						    == NULL)
@@ -1818,7 +1819,7 @@ struct mx *newmxp;
 	if (mxpp == NULL)
 	{
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "linking MX %s(%d) at the top",
+		logg(LOG_DEBUG, "linking MX %s(%d) at the top",
 			newmxp->name, newmxp->pref);
 
 		domp->firstmx = newmxp;
@@ -1826,7 +1827,7 @@ struct mx *newmxp;
 	else
 	{
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "linking MX %s(%d) after MX %s(%d)",
+		logg(LOG_DEBUG, "linking MX %s(%d) after MX %s(%d)",
 			newmxp->name, newmxp->pref,
 			mxpp->name, mxpp->pref);
 
@@ -1857,14 +1858,14 @@ long opt;
 	{
 		/* already registered, drop lower one */
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "duplicated MX %s(%d) found",
+		logg(LOG_DEBUG, "duplicated MX %s(%d) found",
 			mxp->name, mxp->pref);
 
 		if (mxp->pref > pref
 		 || (mxp->pref == pref && mxp->weight > weight))
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "remove it and reuse");
+			logg(LOG_DEBUG, "remove it and reuse");
 
 			if (mxp == dp->firstmx)
 				dp->firstmx = mxp->next;
@@ -1874,7 +1875,7 @@ long opt;
 		else
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "leave it and ignore newer");
+			logg(LOG_DEBUG, "leave it and ignore newer");
 
 			return mxp;	/* do nothing */
 		}
@@ -1886,7 +1887,7 @@ long opt;
 		mxp = (struct mx*)MALLOC(sizeof(struct mx));
 		if (mxp == NULL)
 		{
-			log(LOG_NOTICE, "out of memory (newmx)");
+			logg(LOG_NOTICE, "out of memory (newmx)");
 			return NULL;
 		}
 		bzero(mxp, sizeof(struct mx));
@@ -1923,7 +1924,7 @@ long opt;	/* option described in map file */
 	char msgbuf[MAXLINE], *mxname;
 
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "getmxrr(%s, %s) offset=%d opt=%x",
+	logg(LOG_DEBUG, "getmxrr(%s, %s) offset=%d opt=%x",
 		name, dp->name, offset, opt);
 
 	if (!query_mx)
@@ -1965,7 +1966,7 @@ long opt;	/* option described in map file */
 				&nsin.in.sin_addr);
 		}
 #ifdef INET6
-		else if (inet_pton(INET6, name + 1, &nsin.in6.sin6_addr) > 0)
+		else if (inet_pton(AF_INET6, name + 1, &nsin.in6.sin6_addr) > 0)
 		{
 			/* numeric IPv6 address */
 			name[len-1] = ']';
@@ -2000,7 +2001,7 @@ long opt;	/* option described in map file */
 		qrp = qrp->cname;
 
 	if (cnf.debug & DEBUG_DNS && cnamechain > 0)
-	log(LOG_DEBUG, "CNAME traced: %s", qrp->name);
+	logg(LOG_DEBUG, "CNAME traced: %s", qrp->name);
 
 	if (cnamechain >= CNAMELOOPMAX)
 	{
@@ -2010,7 +2011,7 @@ long opt;	/* option described in map file */
 	}
 
 	if (cnf.debug & DEBUG_DNS)
-	log(LOG_DEBUG, "%s: timeout=%x noex=%x got=%x",
+	logg(LOG_DEBUG, "%s: timeout=%x noex=%x got=%x",
 		qrp->name, qrp->rr_timeout, qrp->rr_noex, qrp->rr_got);
 
 	if (qrp == NULL || qrp->rr_timeout & RR_MX) {
@@ -2045,7 +2046,7 @@ long opt;	/* option described in map file */
 				return -1;
 		}
 		else
-			log(LOG_INFO, "No MX/A on %s for %s; ignored",
+			logg(LOG_INFO, "No MX/A on %s for %s; ignored",
 				name, dp->name);
 		return 0;
 	}
@@ -2090,12 +2091,12 @@ long opt;	/* option described in map file */
 		mxname = cmxp->name;
 #if 0
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "mx=%s pref=%d", mxname, cmxp->pref);
+		logg(LOG_DEBUG, "mx=%s pref=%d", mxname, cmxp->pref);
 #endif
 		if (isamyalias(mxname))
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "found localhost (%s) in MX list",
+			logg(LOG_DEBUG, "found localhost (%s) in MX list",
 				mxname);
 			if (!seenlocal || cmxp->pref < localpref)
 				localpref = cmxp->pref;
@@ -2137,13 +2138,13 @@ long opt;	/* option described in map file */
 			/* already registered, drop lower one */
 
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "duplicated MX %s(%d) found",
+			logg(LOG_DEBUG, "duplicated MX %s(%d) found",
 				mxp->name, mxp->pref);
 
 			if (mxp->pref > offset + cmxp->pref)
 			{
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "remove it for re-link");
+				logg(LOG_DEBUG, "remove it for re-link");
 				mxp->pref = offset + cmxp->pref;
 				if (mxp == dp->firstmx)
 					dp->firstmx = mxp->next;
@@ -2247,13 +2248,13 @@ u_char *t, *b, *e;
 		ap += n;
 		if (ap + INT16SZ*2 > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(class, ap);
 		GETSHORT(type, ap);
 		if (cnf.debug & DEBUG_DNS)
-		log(LOG_DEBUG, "%s: %s(%d,%d), rcode=%d",
+		logg(LOG_DEBUG, "%s: %s(%d,%d), rcode=%d",
 			t, nbuf, class, type, ntohs(hp->rcode));
 	}
 
@@ -2268,7 +2269,7 @@ u_char *t, *b, *e;
 		ap += n;
 		if (ap + INT16SZ*3 + INT32SZ > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		GETSHORT(class, ap);
@@ -2277,13 +2278,13 @@ u_char *t, *b, *e;
 		GETSHORT(n, ap);
 		if (ap + n > eom)
 		{
-			/* log(LOG_NOTICE, "corrupt DNS answer"); */
+			/* logg(LOG_NOTICE, "corrupt DNS answer"); */
 			return;
 		}
 		if (class != C_IN)
 		{
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "DNS: unexpected class %d in an", class);
+			logg(LOG_DEBUG, "DNS: unexpected class %d in an", class);
 			ap += n;
 			continue;
 		}
@@ -2294,20 +2295,20 @@ u_char *t, *b, *e;
 				break;
 			ap += n;
 			if (cnf.debug & DEBUG_DNS)
-			log(LOG_DEBUG, "got MX pref=%d,host=(%s)", pref, nbuf);
+			logg(LOG_DEBUG, "got MX pref=%d,host=(%s)", pref, nbuf);
 #if 0 /* for debug */
 			hep = gethostbyname(nbuf);
 			if (hep == NULL) {
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "get A --  h_errno=%d", h_errno);
+				logg(LOG_DEBUG, "get A --  h_errno=%d", h_errno);
 			} else {
 				if (cnf.debug & DEBUG_DNS)
-				log(LOG_DEBUG, "get A -- OK");
+				logg(LOG_DEBUG, "get A -- OK");
 			}
 #endif
 		} else {
 			ap += n;
-			/* log(LOG_DEBUG, "unexpected answer type: %d", type); */
+			/* logg(LOG_DEBUG, "unexpected answer type: %d", type); */
 		}
 	}
 }
