@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
+ * Copyright (c) 2012 National Institute of Informatics
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +36,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: lmtp.c,v 1.62 2003/08/20 05:12:58 motonori Exp $";
+static char *_id_ = "$Id: lmtp.c,v 1.64 2012/06/07 07:51:35 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -116,7 +117,7 @@ lmtp()
 			if (strcasecmp(lmtpbuf, cmdp->cmd) == 0) {
 
 				if (cnf.debug & DEBUG_LMTP)
-				log(LOG_DEBUG, "LMTP input: %s %s", lmtpbuf,
+				logg(LOG_DEBUG, "LMTP input: %s %s", lmtpbuf,
 					(p == NULL)?"":p);
 				ret = (*cmdp->func)(p);
 				fflush (LmtpOut);
@@ -242,7 +243,7 @@ char *param;
 		return 0;
 	}
 	if (cnf.debug & DEBUG_LMTP)
-	log(LOG_DEBUG, "MAIL FROM: %s size=%s body=%s envid=%s ret=%s", addr,
+	logg(LOG_DEBUG, "MAIL FROM: %s size=%s body=%s envid=%s ret=%s", addr,
 		(env.size == NULL)?"":env.size, (env.body == NULL)?"":env.body,
 		(env.envid == NULL)?"":env.envid, (env.ret == NULL)?"":env.ret);
 	fprintf (LmtpOut, "250 %s Sender ok\n", addr);
@@ -358,7 +359,7 @@ char *param;
 	deliver();		/* delivery with SMTP */
 	lmtp_data_response(1);
 
-	log(LOG_INFO, "nquery=%d/%d nconnect=%d/%d ntimeout=%d ntrans=%d/%d nsent=%d/%d ndeferred=%d nerror=%d+%d dnsmaxcq=%d maxsock=%d tmx=%d tdelivery=%d/%d maxdelay=%d tdelay=%d tsendmail=%d dns_i/o=%d/%d smtp_i/o=%d/%d mem=%d",
+	logg(LOG_INFO, "nquery=%d/%d nconnect=%d/%d ntimeout=%d ntrans=%d/%d nsent=%d/%d ndeferred=%d nerror=%d+%d dnsmaxcq=%d maxsock=%d tmx=%d tdelivery=%d/%d maxdelay=%d tdelay=%d tsendmail=%d dns_i/o=%d/%d smtp_i/o=%d/%d mem=%d",
 		sti.nanswers, sti.nqueries, sti.nquitok, sti.nconnect,
 		sti.ntimeout, sti.noktrans, sti.ntrans, sti.nsent, sti.nrcpt,
 		sti.ndeferred, sti.nnsfailed, sti.nsmtpfailed,
@@ -521,7 +522,7 @@ int finished;	/* response ahead request if finished == 0 */
 			if (strstr(rcptp->response, rcptp->address) == NULL)
 			{
 				if (cnf.debug & DEBUG_LMTP)
-				log(LOG_DEBUG, "%d %s... %s",
+				logg(LOG_DEBUG, "%d %s... %s",
 					rcptp->result, rcptp->address,
 					rcptp->response);
 				fprintf (LmtpOut, "%d %s... %s\n",
@@ -529,7 +530,7 @@ int finished;	/* response ahead request if finished == 0 */
 					rcptp->response);
 			} else {
 				if (cnf.debug & DEBUG_LMTP)
-				log(LOG_DEBUG, "%d %s",
+				logg(LOG_DEBUG, "%d %s",
 					rcptp->result, rcptp->response);
 				fprintf (LmtpOut, "%d %s\n",
 					rcptp->result, rcptp->response);
@@ -542,7 +543,7 @@ int finished;	/* response ahead request if finished == 0 */
 
 			sti.ndeferred++;
 			if (cnf.debug & DEBUG_LMTP)
-			log(LOG_DEBUG, "%d %s... %s",
+			logg(LOG_DEBUG, "%d %s... %s",
 				SMTP_TEMPFAIL(51), rcptp->address, msg);
 			fprintf (LmtpOut, "%d %s... %s\n",
 				SMTP_TEMPFAIL(51), rcptp->address, msg);
@@ -709,7 +710,7 @@ read_message()
 		{
 			msgp->data = (char *)MALLOC(CHUNKSIZE);
 			if (cnf.debug & DEBUG_LMTP)
-			log(LOG_DEBUG, "new chunk %d", ++chunk_num);
+			logg(LOG_DEBUG, "new chunk %d", ++chunk_num);
 			if (msgp->data == NULL)
 			{
 				outofmemory = 1;
@@ -725,7 +726,7 @@ read_message()
 
 	if (outofmemory)
 	{
-		log(LOG_NOTICE, "out of memory (LMTP data buffer allocation)");
+		logg(LOG_NOTICE, "out of memory (LMTP data buffer allocation)");
 		return -1;
 	}
 	return 0;
