@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
  * All rights reserved.
  *
@@ -39,7 +39,7 @@ static char *copyright1 = "@(#)Copyright (c) 1997,1998 Motonori Nakamura";
 static char *copyright2 = "@(#)Copyright (c) 1997,1998 WIDE Project.";
 static char *copyright3 = "@(#)Copyright (c) 1997,1998 Kyoto University.";
 static char *copyright4 = "@(#)All rights reserved.";
-static char *_id_ = "$Id: main.c,v 1.74 2003/08/20 05:12:58 motonori Exp $";
+static char *_id_ = "$Id: main.c,v 1.76 2012/06/07 07:51:35 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -53,6 +53,7 @@ char *progname;
 	printf("Usage: %s [options]\n", progname);
 	printf("Options are:\n");
 	printf("	-1		no piggybacking; par recipient SMTP\n");
+	printf("	-2		separated delivery per domain\n");
 	printf("	-5		check A/AAAA to fallback for v4 or v6 only\n");
 	printf("	-a alias	my alias names to check MX list\n");
 	printf("	-A		try T_ANY query first to get MX\n");
@@ -159,9 +160,9 @@ char *envp[];
 
 	opterr = 0;
 #ifdef INET6
-#define	OPTS	"15Aa:b:c:d:EFg:hI:i:l:M:m:N:n:o:p:q:R:r:S:s:t:uVxyz"
+#define	OPTS	"125Aa:b:c:d:EFg:hI:i:l:M:m:N:n:o:p:q:R:r:S:s:t:uVxyz"
 #else
-#define	OPTS	"15Aa:b:c:d:EFg:hI:i:l:M:m:n:o:p:R:q:r:S:s:t:uVxyz"
+#define	OPTS	"125Aa:b:c:d:EFg:hI:i:l:M:m:n:o:p:R:q:r:S:s:t:uVxyz"
 #endif
 	while ((opt = getopt(argc, argv, OPTS)) != -1)
 	{
@@ -170,6 +171,9 @@ char *envp[];
 		    case '1':	/* 1: no piggybacking */
 			cnf.rcpts_trans = 1;
 			break;
+		    case '2':	/* 2: no piggybacking among domains */
+			cnf.domainsep = 1;
+			break;
 		    case '5':	/* 5: check A/AAAA to fallback for v4/v6 only */
 			cnf.v4v6fallback = 1;
 			break;
@@ -444,7 +448,7 @@ char *envp[];
 		cnf.inetdom |= SMTP_V4|SMTP_V6|SMTP_V6_FIRST;
 #endif
 
-	init_log();
+	init_logg();
 
 	peernamelen = sizeof(peername);
 	if (getpeername(0, (struct sockaddr *)&peername, &peernamelen) == 0)
@@ -455,14 +459,14 @@ char *envp[];
 		switch (peername.in.sin_family)
 		{
 		    case AF_INET:
-			log(LOG_INFO, "connected from %s",
+			logg(LOG_INFO, "connected from %s",
 				inet_ntoa(peername.in.sin_addr));
 			break;
 #ifdef INET6
 		    case AF_INET6:
 			inet_ntop(AF_INET6, &peername.in6.sin6_addr,
 				buf, sizeof(buf));
-			log(LOG_INFO, "connected from %s", buf);
+			logg(LOG_INFO, "connected from %s", buf);
 			break;
 #endif
 		}
@@ -470,7 +474,7 @@ char *envp[];
 
 	if (cnf.sd_max > MAX_SOCK)
 	{
-		log(LOG_INFO, "cnf.sd_max should not be greater than %d, recompile smtpfeed with greater FD_SETSIZE", MAX_SOCK);
+		logg(LOG_INFO, "cnf.sd_max should not be greater than %d, recompile smtpfeed with greater FD_SETSIZE", MAX_SOCK);
 		cnf.sd_max = MAX_SOCK;
 	}
 #ifdef RLIMIT_NOFILE
@@ -478,14 +482,14 @@ char *envp[];
 	{
 		if (limit.rlim_max - 1 < cnf.sd_max)
 		{
-			log(LOG_INFO, "cnf.sd_max should be %d",
+			logg(LOG_INFO, "cnf.sd_max should be %d",
 				limit.rlim_max - 1);
 			cnf.sd_max = limit.rlim_max - 1;
 		}
 		limit.rlim_cur = limit.rlim_max; /* make soft limit the max */
 		if (setrlimit(RLIMIT_NOFILE, &limit) == -1)
 		{
-			log(LOG_NOTICE, "can't set max sd to %d",
+			logg(LOG_NOTICE, "can't set max sd to %d",
 				limit.rlim_cur);
 		}
 	}
@@ -494,18 +498,18 @@ char *envp[];
 #if SHOW_UID
 	uid = geteuid();
 #ifdef INET6
-        log(LOG_DEBUG, "%s (with IPv6) started (uid %d with %d descriptors)",
+        logg(LOG_DEBUG, "%s (with IPv6) started (uid %d with %d descriptors)",
 		version, uid, cnf.sd_max);
 #else
-        log(LOG_DEBUG, "%s started (uid %d with %d descriptors)",
+        logg(LOG_DEBUG, "%s started (uid %d with %d descriptors)",
 		version, uid, cnf.sd_max);
 #endif
 #else
 #ifdef INET6
-        log(LOG_DEBUG, "%s (with IPv6) started (with %d descriptors)",
+        logg(LOG_DEBUG, "%s (with IPv6) started (with %d descriptors)",
 		version, cnf.sd_max);
 #else
-        log(LOG_DEBUG, "%s started (with %d descriptors)",
+        logg(LOG_DEBUG, "%s started (with %d descriptors)",
 		version, cnf.sd_max);
 #endif
 #endif
@@ -551,7 +555,7 @@ char *envp[];
 			}
 		}
 		if (strchr(myname, '.') == NULL)
-			log(LOG_INFO, "hostname (%s) is not a FQDN", myname);
+			logg(LOG_INFO, "hostname (%s) is not a FQDN", myname);
 	}
 	if (addmyalias(myname) < 0)
 	{
@@ -559,13 +563,13 @@ char *envp[];
 	}
 	if (cnf.fallbackmx != NULL && isamyalias(cnf.fallbackmx))
 	{
-		log(LOG_INFO, "ignoring fallbackmx (%s)", cnf.fallbackmx);
+		logg(LOG_INFO, "ignoring fallbackmx (%s)", cnf.fallbackmx);
 		free(cnf.fallbackmx);
 		cnf.fallbackmx = NULL;
 	}
 	if (cnf.pgateway != NULL && isamyalias(cnf.pgateway))
 	{
-		log(LOG_INFO, "ignoring protocol gateway (%s)", cnf.pgateway);
+		logg(LOG_INFO, "ignoring protocol gateway (%s)", cnf.pgateway);
 		free(cnf.pgateway);
 		cnf.pgateway = NULL;
 	}
@@ -574,7 +578,7 @@ char *envp[];
 	{
 		if(rcpts_def_load() < 0)
 		{
-			log(LOG_INFO, "ignoring R option (%s)", cnf.rcpts_def);
+			logg(LOG_INFO, "ignoring R option (%s)", cnf.rcpts_def);
 			free(cnf.rcpts_def);
 			cnf.rcpts_def = NULL;
 		}
