@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
  * All rights reserved.
  *
@@ -35,7 +35,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: smtp.c,v 1.156 2003/08/21 07:04:31 motonori Exp $";
+static char *_id_ = "$Id: smtp.c,v 1.161 2015/02/17 13:59:50 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -54,27 +54,27 @@ sig_alarm()
 {
 	errno = ETIMEDOUT;
 	if (cnf.debug & DEBUG_EVENT)
-	log(LOG_DEBUG, "alarm clock");
+	logg(LOG_DEBUG, "alarm clock");
 }
 
 static void
 sig_int()
 {
-	log(LOG_INFO, "SIGINT received: internal states dumped");
+	logg(LOG_INFO, "SIGINT received: internal states dumped");
 	dump_internal();
 }
 
 static void
 sig_hup()
 {
-	log(LOG_INFO, "SIGHUP received: graceful termination");
+	logg(LOG_INFO, "SIGHUP received: graceful termination");
 	forced_terminate();
 }
 
 static void
 sig_term()
 {
-	log(LOG_INFO, "SIGTERM received: semi graceful termination");
+	logg(LOG_INFO, "SIGTERM received: semi graceful termination");
 	markstatus(env.rcpt_list, SMTP_TEMPFAIL(51),
 		"Aborted by INT signal", 1);
 	longjmp(jmp_env, 1);
@@ -87,10 +87,10 @@ deliver()
 	int s;
 
 	while((s = dup(0)) < 253);
-	log(LOG_DEBUG, "descriptor reserved till %d", s);
+	logg(LOG_DEBUG, "descriptor reserved till %d", s);
 #endif
 
-	log(LOG_INFO, "from=%s size=%d nrcpt=%d msgid=%s",
+	logg(LOG_INFO, "from=%s size=%d nrcpt=%d msgid=%s",
 		env.sender, env.realsize, sti.nrcpt,
 		(env.mid == NULL)?"none":env.mid);
 
@@ -99,7 +99,7 @@ deliver()
 		FILE *fp;
 
 		if (cnf.debug & DEBUG_TRANS)
-			log(LOG_DEBUG, "searching sender address %s in %s",
+			logg(LOG_DEBUG, "searching sender address %s in %s",
 				env.sender, cnf.src_mod_path);
 		if ((fp = fopen(cnf.src_mod_path, "r")) == NULL)
 		{
@@ -127,7 +127,7 @@ deliver()
 			fclose(fp);
 			if (found)
 			{
-				log(LOG_INFO,
+				logg(LOG_INFO,
 					"recipients/transaction changed to 1");
 				cnf.rcpts_trans = 1;
 			}
@@ -357,13 +357,13 @@ smtp()
 		if (rcptp->domain == NULL)
 		{
 			rcptp->stat = RCPT_DONE;
-			log(LOG_NOTICE, "no domain for %s", rcptp->address);
+			logg(LOG_NOTICE, "no domain for %s", rcptp->address);
 			continue;
 		}
 		if (rcptp->domain->firstmx == NULL)
 		{
 			rcptp->stat = RCPT_DONE;
-			log(LOG_NOTICE, "no MX for %s", rcptp->address);
+			logg(LOG_NOTICE, "no MX for %s", rcptp->address);
 			continue;
 		}
 	}
@@ -376,7 +376,7 @@ smtp()
 		if (domp->rcpt_top == NULL)
 			continue;
 		if (cnf.debug & DEBUG_TRANS)
-		log(LOG_DEBUG, "pick up domain: %s", domp->name);
+		logg(LOG_DEBUG, "pick up domain: %s", domp->name);
 		if (wdomp == NULL)
 			env.work_domain = domp;
 		else
@@ -396,7 +396,7 @@ smtp()
 		{
 			/* all recipients had been processed */
 			if (cnf.debug & DEBUG_CONNECT)
-			log(LOG_DEBUG, "closing all sockets");
+			logg(LOG_DEBUG, "closing all sockets");
 			all_done = 1;
 			for (i = 0; i < cnf.sd_max; i++)
 			{
@@ -425,7 +425,7 @@ smtp()
 			}
 
 			if (cnf.debug & DEBUG_TRANS)
-			log(LOG_DEBUG, "in_use=%d, max_sock=%d, min_sock=%d",
+			logg(LOG_DEBUG, "in_use=%d, max_sock=%d, min_sock=%d",
 				in_use, max_sock, sti.minsock);
 
 			if (in_use > (max_sock - sti.minsock))
@@ -440,7 +440,7 @@ smtp()
 					if (connwork[i].host->state != STAT_SENT)
 						continue;
 					if (cnf.debug & DEBUG_CONNECT)
-					log(LOG_DEBUG, "%d/%d: closing idle socket",
+					logg(LOG_DEBUG, "%d/%d: closing idle socket",
 						connwork[i].seq,
 						connwork[i].socket);
 					smtp_quit_s(&connwork[i]);
@@ -459,7 +459,7 @@ smtp()
 				if (!connwork[i].reuse)
 					break;
 				if (cnf.debug & DEBUG_CONNECT)
-				log(LOG_DEBUG, "%d/%d: closing idle socket",
+				logg(LOG_DEBUG, "%d/%d: closing idle socket",
 					connwork[i].seq,
 					connwork[i].socket);
 				smtp_quit_s(&connwork[i]);
@@ -480,7 +480,7 @@ smtp()
 			}
 #if 1
 			if (cnf.debug & DEBUG_TRANS)
-			log(LOG_DEBUG, "new transaction: trans=%d, sock=%d",
+			logg(LOG_DEBUG, "new transaction: trans=%d, sock=%d",
 				num_trans, num_sock);
 #endif
 			if (new_transaction() < 0)
@@ -515,7 +515,7 @@ smtp()
 		if (all_done && fds < 0)
 		{
 			if (cnf.debug & DEBUG_CONNECT)
-			log(LOG_DEBUG, "all sockets are closed");
+			logg(LOG_DEBUG, "all sockets are closed");
 #if 0
 			dump_internal();
 #endif
@@ -524,7 +524,7 @@ smtp()
 
 		if (fds < 0)	/* XXX */
 		{
-			log(LOG_INFO, "No active sockets");
+			logg(LOG_INFO, "No active sockets");
 			if (bad_state++ > 5)
 			{
 				markstatus(env.rcpt_list, SMTP_TEMPFAIL(51),
@@ -562,12 +562,12 @@ smtp()
 		{
 			/* time up */
 			forced_terminate();
-			log(LOG_INFO, "total processing timed out (%d sec)",
+			logg(LOG_INFO, "total processing timed out (%d sec)",
 				cnf.t_timeout);
 		}
 		if (rc < 0)
 		{
-			log(LOG_INFO, "select returns %d (fds=%d, errno=%d)",
+			logg(LOG_INFO, "select returns %d (fds=%d, errno=%d)",
 				rc, fds, errno);
 			sleep(1);
 		}
@@ -595,7 +595,7 @@ new_transaction()
 	{ 
 #if 1
 		if (cnf.debug & DEBUG_TRANS)
-		log(LOG_DEBUG, "checking domain: %s", domp->name);
+		logg(LOG_DEBUG, "checking domain: %s", domp->name);
 #endif
 		/* first time for the domain? */
 #if 1
@@ -616,7 +616,7 @@ new_transaction()
 		      || domp->curmx->host->addr == NULL))))
 		{
 			if (domp->curmx->host == NULL)
-				log(LOG_INFO, "skipping bad MX: %s for %s",
+				logg(LOG_INFO, "skipping bad MX: %s for %s",
 					domp->curmx->name, domp->name);
 			else
 			{
@@ -698,7 +698,7 @@ new_transaction()
 	}
 #if 1
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "no new transaction");
+	logg(LOG_DEBUG, "no new transaction");
 #endif
 	NewTransCheck = 0;
 	return -1;	/* no new transaction */
@@ -722,7 +722,7 @@ new_transaction()
 
 		snprintf(msgbuf, sizeof(msgbuf),
 			"No address known for %s", conn->host->name);
-		log(LOG_INFO, msgbuf, conn->host->name);
+		logg(LOG_INFO, msgbuf, conn->host->name);
 
 		rcode = SMTP_TEMPFAIL(52);
 		if (cnf.absentAok)
@@ -799,7 +799,7 @@ int timeout_only, true_term;
 			/* timeout! */
 			sti.ntimeout++;
 			if (cnf.debug & DEBUG_CONNECT)
-			log(LOG_DEBUG, "%d/%d: timeout (%s)", conn->seq,
+			logg(LOG_DEBUG, "%d/%d: timeout (%s)", conn->seq,
 				conn->socket, STATE(st));
 			if (st == STAT_DTERMSENT)
 			{
@@ -815,7 +815,7 @@ int timeout_only, true_term;
 		if (FD_ISSET(conn->socket, w_fds))
 		{
 			if (cnf.debug & DEBUG_EVENT)
-			log(LOG_DEBUG, "%d/%d: writable (%s)",
+			logg(LOG_DEBUG, "%d/%d: writable (%s)",
 				conn->seq, conn->socket, STATE(st));
 			if (state_tbl[st].send != NULL)
 			{
@@ -826,14 +826,14 @@ int timeout_only, true_term;
 			if (conn->host == NULL)
 			{
 				if (cnf.debug & DEBUG_EVENT)
-				log(LOG_DEBUG, "%d/%d: state %s -> NULL",
+				logg(LOG_DEBUG, "%d/%d: state %s -> NULL",
 					conn->seq, conn->socket, STATE(st));
 			}
 			else if (st != conn->host->state)
 			{
 				conn->host->time = Now;
 				if (cnf.debug & DEBUG_EVENT)
-				log(LOG_DEBUG, "%d/%d: state %s -> %s",
+				logg(LOG_DEBUG, "%d/%d: state %s -> %s",
 					conn->seq, conn->socket, STATE(st),
 					STATE(conn->host->state));
 			}
@@ -842,7 +842,7 @@ int timeout_only, true_term;
 		if (FD_ISSET(conn->socket, r_fds))
 		{
 			if (cnf.debug & DEBUG_EVENT)
-			log(LOG_DEBUG, "%d/%d: readable (%s)",
+			logg(LOG_DEBUG, "%d/%d: readable (%s)",
 				conn->seq, conn->socket, STATE(st));
 			if (state_tbl[st].recv != NULL)
 			{
@@ -853,14 +853,14 @@ int timeout_only, true_term;
 			if (conn->host == NULL)
 			{
 				if (cnf.debug & DEBUG_EVENT)
-				log(LOG_DEBUG, "%d/%d: state %s -> NULL",
+				logg(LOG_DEBUG, "%d/%d: state %s -> NULL",
 					conn->seq, conn->socket, STATE(st));
 			}
 			else if (st != conn->host->state)
 			{
 				conn->host->time = Now;
 				if (cnf.debug & DEBUG_EVENT)
-				log(LOG_DEBUG, "%d/%d: state %s -> %s",
+				logg(LOG_DEBUG, "%d/%d: state %s -> %s",
 					conn->seq, conn->socket, STATE(st),
 					STATE(conn->host->state));
 			}
@@ -885,7 +885,7 @@ int tracechain;
 	if (rcptp == NULL)
 	{
 		if (cnf.debug & DEBUG_TRANS)
-		log(LOG_DEBUG, "markstatus: rcptp is NULL");
+		logg(LOG_DEBUG, "markstatus: rcptp is NULL");
 		return;
 	}
 
@@ -1047,7 +1047,7 @@ int tracechain;
 		{
 			/* buffer may be full */
 			strcat(addrbuf+addrlen, ",(more...)");
-			log(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",
+			logg(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",
 				sti.nsent, sti.ndeferred, sti.nnsfailed,
 				sti.nsmtpfailed, sti.nrcpt, hostname,
 				addrbuf, proto, delay, rcode,
@@ -1069,7 +1069,7 @@ int tracechain;
 			{
 				rcptp->stat = RCPT_RETRY;
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "tempfail; retry: %s",
+				logg(LOG_DEBUG, "tempfail; retry: %s",
 					rcptp->address);
 			}
 			else if (!Terminate && mxp != NULL
@@ -1079,7 +1079,7 @@ int tracechain;
 			{
 				rcptp->stat = RCPT_RETRY;
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "tempfail; next avail: %s",
+				logg(LOG_DEBUG, "tempfail; next avail: %s",
 					rcptp->address);
 			}
 			else
@@ -1087,7 +1087,7 @@ int tracechain;
 				sti.ndeferred++;
 				rcptp->stat = RCPT_DONE;
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "tempfail; done: %s",
+				logg(LOG_DEBUG, "tempfail; done: %s",
 					rcptp->address);
 			}
 		}
@@ -1116,7 +1116,7 @@ int tracechain;
 	}
 	if (addrlen > 0)
 	{
-		log(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",
+		logg(LOG_INFO, "(%d+%d+%d+%d/%d) relay=%s to=%s proto=%s delay=%d code=%d (%s)",
 			sti.nsent, sti.ndeferred, sti.nnsfailed,
 			sti.nsmtpfailed, sti.nrcpt, hostname,
 			addrbuf, proto, delay, rcode,
@@ -1156,7 +1156,7 @@ struct recipient *rcptp;
 
 #if 1
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "num_trans=%d, num_sock=%d, max_sock=%d, min_sock=%d",
+	logg(LOG_DEBUG, "num_trans=%d, num_sock=%d, max_sock=%d, min_sock=%d",
 		num_trans, num_sock, max_sock, sti.minsock);
 #endif
 	if (num_trans > (max_sock - sti.minsock))
@@ -1171,7 +1171,7 @@ struct recipient *rcptp;
 
 #if 1
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "open transaction for %s", rcptp->address);
+	logg(LOG_DEBUG, "open transaction for %s", rcptp->address);
 #endif
 
 	if (rcptp->domain->curmx->host->state == STAT_SENT)
@@ -1183,7 +1183,7 @@ struct recipient *rcptp;
 				break;
 		}
 		if (cnf.debug & DEBUG_TRANS)
-		log(LOG_DEBUG, "reusing: slot=%d", i);
+		logg(LOG_DEBUG, "reusing: slot=%d", i);
 	}
 	else
 	{
@@ -1203,7 +1203,7 @@ struct recipient *rcptp;
 	{
 #if 0
 		if (cnf.debug & DEBUG_TRANS)
-		log(LOG_DEBUG, "no free slot");
+		logg(LOG_DEBUG, "no free slot");
 #endif
 		/* sleep (1); */
 		return NULL;	/* XXX socket slot is full */
@@ -1212,7 +1212,7 @@ struct recipient *rcptp;
 	sti.ntrans++;
 
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "%d/-: open transaction for %s, slot=%d",
+	logg(LOG_DEBUG, "%d/-: open transaction for %s, slot=%d",
 		sti.ntrans, rcptp->address, i);
 
 	/* initialize transaction */
@@ -1246,7 +1246,7 @@ struct recipient *rcptp;
 		domp = mxp->domain;
 		if (domp == NULL)
 		{
-			log(LOG_NOTICE, "Oops, mxp->domain for %s is NULL",
+			logg(LOG_NOTICE, "Oops, mxp->domain for %s is NULL",
 				mxp->name);
 			goto next_domain;
 		}
@@ -1269,7 +1269,7 @@ struct recipient *rcptp;
 				if (rcpts + 1 > cnf.rcpts_trans)
 #else
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "check count of rcpts (domain:%s, max rcpts:%d, count:%d)",
+				logg(LOG_DEBUG, "check count of rcpts (domain:%s, max rcpts:%d, count:%d)",
 					rcptp2->domain->name,
 					rcptp2->domain->rcpts,
 					rcpts);
@@ -1282,7 +1282,7 @@ struct recipient *rcptp;
 				}
 				rcpts++;
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "%d/-: with %s",
+				logg(LOG_DEBUG, "%d/-: with %s",
 					sti.ntrans, rcptp2->address);
 				rcptp2->stat = RCPT_WORK;
 				rcptp2->chain = NULL;
@@ -1292,6 +1292,11 @@ struct recipient *rcptp;
 			}
 		}
   next_domain:
+		if (cnf.domainsep) {
+			if (cnf.debug & DEBUG_TRANS)
+			logg(LOG_DEBUG, "skip piggybacking with other domains");
+			goto rcpts_max;
+		}
 		if (first_time)
 		{
 			first_time = 0;
@@ -1323,7 +1328,7 @@ int next;	/* true: forward MX for next trial */
 #endif
 
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "%d/%d: close transaction", conn->seq, conn->socket);
+	logg(LOG_DEBUG, "%d/%d: close transaction", conn->seq, conn->socket);
 
 	NewTransCheck = 1;		/* do not skip new_transaction() */
 	env.work_dom_ptr = NULL;	/* retry from the top of list */
@@ -1388,7 +1393,7 @@ int next;	/* true: forward MX for next trial */
 			if (domp->curmx != NULL)
 			{
 				if (cnf.debug & DEBUG_TRANS)
-				log(LOG_DEBUG, "%d/-: switching to next MX: %s",
+				logg(LOG_DEBUG, "%d/-: switching to next MX: %s",
 					conn->seq, domp->curmx->name);
 			}
 		}
@@ -1446,7 +1451,7 @@ struct domain *domp;
 		{
 			/* remove recipient from working chain */
 			if (cnf.debug & DEBUG_TRANS)
-			log(LOG_DEBUG, "removing recipient: %s",
+			logg(LOG_DEBUG, "removing recipient: %s",
 				rcptp->address);
 			if (rcptp == domp->rcpt_top)
 			{
@@ -1483,7 +1488,7 @@ trim_domain()
 		{
 			/* remove from working domain chain */
 			if (cnf.debug & DEBUG_TRANS)
-			log(LOG_DEBUG, "removing domain: %s",
+			logg(LOG_DEBUG, "removing domain: %s",
 				domp->name);
 			if (domp == env.work_domain)
 			{
@@ -1538,7 +1543,7 @@ struct connection *conn;
 
 	if (conn->host == NULL)
 	{
-		log(LOG_NOTICE, "smtp_connect called with NULL host");
+		logg(LOG_NOTICE, "smtp_connect called with NULL host");
 		return -1;
 	}
 
@@ -1552,7 +1557,7 @@ struct connection *conn;
 	}
 
 	if (cnf.debug & DEBUG_CONNECT)
-	log(LOG_DEBUG, "%d/-: connecting to %s",
+	logg(LOG_DEBUG, "%d/-: connecting to %s",
 		conn->seq, conn->host->name);
 
 	bzero((char *)&rsin, sizeof(rsin));
@@ -1589,7 +1594,7 @@ struct connection *conn;
 			socksize = sizeof(struct sockaddr_in);
 
 			if (cnf.debug & DEBUG_CONNECT)
-			log(LOG_DEBUG, "%d/-: destination=%s/%d", conn->seq,
+			logg(LOG_DEBUG, "%d/-: destination=%s/%d", conn->seq,
 				inet_ntoa(rsin.in.sin_addr), cnf.dst_port);
 			found = 1;
 			break;
@@ -1619,7 +1624,7 @@ struct connection *conn;
 
 				inet_ntop(AF_INET6, &rsin.in6.sin6_addr,
 					buf, sizeof(buf));
-				log(LOG_DEBUG, "%d/-: destination=%s/%d",
+				logg(LOG_DEBUG, "%d/-: destination=%s/%d",
 					conn->seq, buf, cnf.dst_port);
 			}
 			found = 1;
@@ -1657,7 +1662,7 @@ struct connection *conn;
 
 	if (sd > MAX_SOCK)
 	{
-		log(LOG_NOTICE, "%d/%d: can not manage such a big file descriptor number with select()",
+		logg(LOG_NOTICE, "%d/%d: can not manage such a big file descriptor number with select()",
 			conn->seq, sd);
 		close(sd);
 		if (max_sock > MAX_SOCK)
@@ -1670,7 +1675,7 @@ struct connection *conn;
 
 	if (sd < 0){
 		/* no more socket resource */
-		log(LOG_NOTICE, "socket allocation failed for %s: %s (may be max=%d, current=%d/%d/%d)",
+		logg(LOG_NOTICE, "socket allocation failed for %s: %s (may be max=%d, current=%d/%d/%d)",
 			conn->host->name, smtpstrerror(errno), sti.maxsock,
 			max_sock, num_sock, sti.minsock);
 		if (max_sock > NOFILE)
@@ -1689,14 +1694,14 @@ struct connection *conn;
 	}
 
 	if (cnf.debug & DEBUG_CONNECT)
-	log(LOG_DEBUG, "%d/%d: socket allocated", conn->seq, sd);
+	logg(LOG_DEBUG, "%d/%d: socket allocated", conn->seq, sd);
 
 #ifdef SO_SNDBUF
 	if (cnf.sendbufsize > 0)
 	{
 		if (setsockopt(sd, SOL_SOCKET, SO_SNDBUF,
 		    (char *)&cnf.sendbufsize, sizeof(cnf.sendbufsize)) < 0)
-			log(LOG_NOTICE, "%d/%d: setsockopt(SO_SNDBUF) failed for %s: %s",
+			logg(LOG_NOTICE, "%d/%d: setsockopt(SO_SNDBUF) failed for %s: %s",
 				conn->seq, sd, conn->host->name,
 				smtpstrerror(errno));
 		/* ignore error */
@@ -1735,7 +1740,7 @@ struct connection *conn;
 					break;
 #ifdef INET6
 				    case AF_INET6:
-					if (inet_pton(INET6, p,
+					if (inet_pton(AF_INET6, p,
 						      &lsin.in6.sin6_addr) > 0)
 						found = 1;
 					break;
@@ -1761,7 +1766,7 @@ struct connection *conn;
 				    case AF_INET:
 					bcopy(addrp->address,
 						&lsin.in.sin_addr, addrp->len);
-					log(LOG_INFO,
+					logg(LOG_INFO,
 						"using %s as source IP address",
 						inet_ntoa(lsin.in.sin_addr));
 					break;
@@ -1773,7 +1778,7 @@ struct connection *conn;
 
 					inet_ntop(AF_INET6, &lsin.in6.sin6_addr,
 						buf, sizeof(buf));
-					log(LOG_INFO, "using %s as source IP address", buf);
+					logg(LOG_INFO, "using %s as source IP address", buf);
 					break;
 #endif
 				}
@@ -1791,7 +1796,7 @@ struct connection *conn;
 				p = NULL;
 		}
 		if (!found)
-			log(LOG_INFO, "sourceIP (%s) ignored", cnf.sourceIP);
+			logg(LOG_INFO, "sourceIP (%s) ignored", cnf.sourceIP);
 	}
 	if (!found)
 	{
@@ -1840,7 +1845,7 @@ struct connection *conn;
 	{
 		if (bind(sd, (struct sockaddr *)&lsin, socksize) < 0)
 		{
-			log(LOG_NOTICE, "%d/%d: bind failed for %s: %s", 
+			logg(LOG_NOTICE, "%d/%d: bind failed for %s: %s", 
 				conn->seq, sd, conn->host->name,
 				smtpstrerror(errno));
 			conn->host->state = STAT_FAIL;
@@ -1857,7 +1862,7 @@ struct connection *conn;
 
 	if (ioctl (sd, FIONBIO, &res) < 0)
 	{
-		log(LOG_NOTICE, "%d/%d: FIONBIO failed for %s: %s",
+		logg(LOG_NOTICE, "%d/%d: FIONBIO failed for %s: %s",
 			conn->seq, sd, conn->host->name, smtpstrerror(errno));
 		conn->host->state = STAT_FAIL;
 		close (sd);
@@ -1868,7 +1873,7 @@ struct connection *conn;
 #else
 	if ((res = fcntl(sd, F_GETFL, 0)) == -1)
 	{
-		log(LOG_NOTICE, "%d/%d: F_GETFL failed for %s: %s",
+		logg(LOG_NOTICE, "%d/%d: F_GETFL failed for %s: %s",
 			conn->seq, sd, conn->host->name, smtpstrerror(errno));
 		conn->host->state = STAT_FAIL;
 		close (sd);
@@ -1883,7 +1888,7 @@ struct connection *conn;
 # endif
 	if (fcntl(sd, F_SETFL, res) == -1)
 	{
-		log(LOG_NOTICE, "%d/%d: F_SETFL failed for %s: %s",
+		logg(LOG_NOTICE, "%d/%d: F_SETFL failed for %s: %s",
 			conn->seq, sd, conn->host->name, smtpstrerror(errno));
 		conn->host->state = STAT_FAIL;
 		close (sd);
@@ -1899,7 +1904,7 @@ struct connection *conn;
 	 && errno != EINPROGRESS && errno != EINTR)	/* XXX */
 	{
 		alarm(0);
-		log(LOG_DEBUG, "%d/%d: connect failed for %s: %s",
+		logg(LOG_DEBUG, "%d/%d: connect failed for %s: %s",
 			conn->seq, sd, conn->host->name, smtpstrerror(errno));
 		conn->host->current->stat = EX_TEMPFAIL;
 		close (sd);
@@ -1931,7 +1936,7 @@ struct connection *conn;
 		}
 		sti.minsock = sd;
 		if (cnf.debug & DEBUG_CONNECT)
-		log(LOG_DEBUG, "minsock=%d", sti.minsock);
+		logg(LOG_DEBUG, "minsock=%d", sti.minsock);
 	}
 	num_sock++;
 	return 0;
@@ -1955,7 +1960,7 @@ struct connection *conn;
 
 	if (getpeername(conn->socket, (struct sockaddr *)&cin, &len) < 0)
 	{
-		log(LOG_INFO, "%d/%d: getpeername failed for %s: %s",
+		logg(LOG_INFO, "%d/%d: getpeername failed for %s: %s",
 			conn->seq, conn->socket, conn->host->name,
 			smtpstrerror(errno));
 		if (conn->host->current != NULL) /* XXX for safe access */
@@ -2004,7 +2009,7 @@ struct connection *conn;
 	conn->outlen = 0;
 	if (conn->inbuf == NULL || conn->outbuf == NULL)
 	{
-		log(LOG_NOTICE, "out of memory (SMTP buffer allocation)");
+		logg(LOG_NOTICE, "out of memory (SMTP buffer allocation)");
 		if (conn->host->current != NULL) /* XXX for safe access */
 		{
 			conn->host->current->stat = EX_TEMPFAIL;
@@ -2048,7 +2053,7 @@ int *rest;
 			if (n == 0)
 			{
 				if (cnf.debug & DEBUG_CONNECT)
-				log(LOG_DEBUG, "%d/%d: closed by foreign host",
+				logg(LOG_DEBUG, "%d/%d: closed by foreign host",
 					conn->seq, conn->socket);
 				errno = E_SF_IO;
 				return -1;
@@ -2066,7 +2071,7 @@ int *rest;
 #endif
 				return 0;
 			  default:
-				log(LOG_DEBUG, "%d/%d: read failed for %s: ret=%d, errno=%d",
+				logg(LOG_DEBUG, "%d/%d: read failed for %s: ret=%d, errno=%d",
 					conn->seq, conn->socket,
 					conn->host->name, n, errno);
 				return n;
@@ -2132,7 +2137,7 @@ int flush;
 			if (n == 0)
 			{
 				if (cnf.debug & DEBUG_CONNECT)
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"%d/%d: closed by foreign host (%s)",
 					conn->seq, conn->socket,
 					conn->host->name);
@@ -2152,7 +2157,7 @@ int flush;
 #endif
 				return 0;
 			  default:
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"%d/%d: write failed for %s: ret=%d, errno=%d",
 					conn->seq, conn->socket,
 					conn->host->name, n, errno);
@@ -2187,7 +2192,7 @@ int flush;
 			if (n == 0)
 			{
 				if (cnf.debug & DEBUG_CONNECT)
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"%d/%d: closed by foreign host (%s)",
 					conn->seq, conn->socket,
 					conn->host->name);
@@ -2205,7 +2210,7 @@ int flush;
 #endif
 				return size;
 			  default:
-				log(LOG_DEBUG,
+				logg(LOG_DEBUG,
 					"%d/%d: write failed for %s: ret=%d, errno=%d",
 					conn->seq, conn->socket,
 					conn->host->name, n, errno);
@@ -2264,9 +2269,9 @@ int *rest;
 	char *cbuf = firstline;
 	int buflen = size;
 	char contbuf[MAXLINE];
-	char savec;
 	int len, rcode, savestate;
 	int stat = 0;
+	int has_scode = 0;
 
 	cbuf[0] = '\0';
 	savestate = conn->host->state;
@@ -2301,18 +2306,39 @@ int *rest;
 			cbuf[--len] = '\0';
 		}
 		if (cnf.debug & DEBUG_SMTP)
-		log(LOG_DEBUG, "%d/%d: >>> %s", conn->seq, conn->socket, cbuf);
+		logg(LOG_DEBUG, "%d/%d: >>> %s", conn->seq, conn->socket, cbuf);
 		if (func != NULL)
 			(*func)(conn, cbuf);
 		if (!isdigit(cbuf[0]) || !isdigit(cbuf[1]) || !isdigit(cbuf[2])
 		 || (cbuf[3] != '\0' && !isspace(cbuf[3]) && cbuf[3] != '-'))
 			continue;
-		savec = cbuf[3];
 		if (cbuf[0] >= '1' && cbuf[0] <= '5')
 			stat = NUM(cbuf[0])*100 + NUM(cbuf[1])*10
 			     + NUM(cbuf[2]);
+		if (cbuf != firstline) {
+			int pi = 3;
+
+			if (has_scode > 0
+			 && strncmp(firstline + 4, cbuf + 4, has_scode - 4) == 0)
+				pi = has_scode;
+			if (strlen(firstline) + strlen(&cbuf[pi]) < size) {
+				char savec = cbuf[pi];
+
+				cbuf[pi] = ' ';
+				strncat(firstline, &cbuf[pi], size);
+				cbuf[pi] = savec;
+			}
+		}
 		if (cbuf[3] != '-')
 			break;
+		if (cbuf == firstline) {
+			int pi = 4;
+			while (firstline[pi] != '\0'
+			 && (isdigit(firstline[pi]) || firstline[pi] == '.'))
+				pi++;
+			if (firstline[pi] != '\0' && isspace(firstline[pi]))
+				has_scode = pi;
+		}
 		cbuf = contbuf;
 		buflen = sizeof(contbuf);
 	}
@@ -2358,7 +2384,7 @@ int flush;
 	int len = strlen(buf);
 
 	if (cnf.debug & DEBUG_SMTP)
-	log(LOG_DEBUG, "%d/%d: <<< %s", conn->seq, conn->socket, buf);
+	logg(LOG_DEBUG, "%d/%d: <<< %s", conn->seq, conn->socket, buf);
 	if (len + 2 < size)
 	{
 		strcat(buf, CRLF);
@@ -2473,7 +2499,7 @@ struct connection *conn;
 			errmsg = "Unexpected Error";
 			break;
 		}
-		log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+		logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 			conn->seq, conn->socket, conn->host->name,
 			smtpstrerror(errno), STATE(savestate));
 		if (conn->host->current != NULL) /* XXX for safe access */
@@ -2620,7 +2646,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, smtpstrerror(errno),
 				conn->host->name, STATE(savestate));
 		}
@@ -2876,7 +2902,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3056,7 +3082,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3208,7 +3234,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3323,7 +3349,7 @@ struct connection *conn;
 		len = strlen(altbuf);
 		rcode = smtp_write(conn, altbuf, len, 0);
 		if (rcode != len)	/* XXX */
-			log(LOG_NOTICE,
+			logg(LOG_NOTICE,
 			    "Header of sent message will be corrupted: %s",
 			    altbuf);
 	}
@@ -3395,7 +3421,7 @@ struct connection *conn;
 		return 0;
 
 	if (cnf.debug & DEBUG_CONNECT)
-	log(LOG_DEBUG, "reusability check: %s", conn->host->name);
+	logg(LOG_DEBUG, "reusability check: %s", conn->host->name);
 
 	/* check possibility of reuse connection */
 	for (rmxp = conn->host->mx_ref; rmxp != NULL; rmxp = rmxp->mx_ref)
@@ -3417,7 +3443,7 @@ struct connection *conn;
 			}
 		}
 		if (cnf.debug & DEBUG_CONNECT)
-		log(LOG_DEBUG, "on domain: %s = %d", domp->name, reused);
+		logg(LOG_DEBUG, "on domain: %s = %d", domp->name, reused);
 
 		if (reused == 0)
 			continue;
@@ -3429,7 +3455,7 @@ struct connection *conn;
 			if (mxp == rmxp)
 			{
 				if (cnf.debug & DEBUG_CONNECT)
-				log(LOG_DEBUG, "%d/%d: may be reused: %s",
+				logg(LOG_DEBUG, "%d/%d: may be reused: %s",
 					conn->seq, conn->socket,
 					conn->host->name);
 				return 1;
@@ -3455,7 +3481,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3552,7 +3578,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3629,7 +3655,7 @@ struct connection *conn;
 	{
 		if (errno > 0)
 		{
-			log(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
+			logg(LOG_INFO, "%d/%d: failed for %s: %s (%s)",
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
@@ -3691,7 +3717,7 @@ struct connection *conn;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
 				newstr(buf), 1);
 		else
-			log(LOG_INFO, "Trying next address of %s",
+			logg(LOG_INFO, "Trying next address of %s",
 			    conn->host->name);
 		if (conn->host->current != NULL) /* XXX for safe access */
 			conn->host->current = conn->host->current->next;
@@ -3717,7 +3743,7 @@ struct connection *conn;
 		{
 			conn->host->state = STAT_CLOSED;
 			conn->host->current->stat = EX_TEMPFAIL;
-			log(LOG_INFO, "Trying next address of %s",
+			logg(LOG_INFO, "Trying next address of %s",
 			    conn->host->name);
 		}
 		else
@@ -3744,17 +3770,17 @@ dump_internal()
 	struct recipient *rcptp;
 	int i;
 
-	log(LOG_INFO, "=== dumping internal information ===");
+	logg(LOG_INFO, "=== dumping internal information ===");
 
 	for (rcptp = env.rcpt_list; rcptp != NULL; rcptp = rcptp->next)
 	{
 		switch (rcptp->stat)
 		{
 		    case RCPT_NOOP:
-			log(LOG_INFO, "rcpt=%s stat=noop", rcptp->address);
+			logg(LOG_INFO, "rcpt=%s stat=noop", rcptp->address);
 			break;
 		    case RCPT_WORK:
-			log(LOG_INFO, "rcpt=%s stat=working mx=%s st=%s",
+			logg(LOG_INFO, "rcpt=%s stat=working mx=%s st=%s",
 				rcptp->address,
 				(rcptp->domain->curmx == NULL)?"none":
 				rcptp->domain->curmx->name,
@@ -3763,18 +3789,18 @@ dump_internal()
 			break;
 		    case RCPT_RETRY:
 			if (rcptp->domain->curmx != NULL)
-				log(LOG_INFO,
+				logg(LOG_INFO,
 				    "rcpt=%s stat=retry code=%d mx=%s st=%s",
 				    rcptp->address, rcptp->result,
 				    rcptp->domain->curmx->name,
 				    STATE(rcptp->domain->curmx->host->state));
 			else
-				log(LOG_INFO,
+				logg(LOG_INFO,
 					"rcpt=%s stat=retry code=%d mx=none",
 					rcptp->address, rcptp->result);
 			break;
 		    case RCPT_DONE:
-			log(LOG_INFO, "rcpt=%s stat=done code=%d",
+			logg(LOG_INFO, "rcpt=%s stat=done code=%d",
 				rcptp->address, rcptp->result);
 			break;
 		}
@@ -3784,12 +3810,12 @@ dump_internal()
 	{
 		if (connwork[i].host == NULL)
 			continue;
-		log(LOG_INFO, "slot=%d host=%s stat=%s", i,
+		logg(LOG_INFO, "slot=%d host=%s stat=%s", i,
 			connwork[i].host->name,
 			STATE(connwork[i].host->state));
 	}
 
-	log(LOG_INFO, "=== end of internal information ===");
+	logg(LOG_INFO, "=== end of internal information ===");
 }
 
 static char *
