@@ -6,7 +6,7 @@
 
 $LowLimit = 10;
 
-%month = ('Jan', 1, 'Feb', 2, 'Mar', 3, 'Apr', 4, 'Mar', 5, 'Jun', 6,
+%month = ('Jan', 1, 'Feb', 2, 'Mar', 3, 'Apr', 4, 'May', 5, 'Jun', 6,
 	'Jul', 7, 'Aug', 8, 'Sep', 9, 'Oct', 10, 'Nov', 11, 'Dec', 12);
 
 print "# Version 1\n";
