@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2012 Motonori Nakamura <motonori@wide.ad.jp>
+ * Copyright (c) 1997-2012 WIDE Project
  * Copyright (c) 1997-2003 Kyoto University
+ * Copyright (c) 2012 National Institute of Informatics
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +36,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: util.c,v 1.21 2003/08/20 05:12:58 motonori Exp $";
+static char *_id_ = "$Id: util.c,v 1.23 2012/06/07 07:51:35 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -49,7 +50,7 @@ char *str;
 
 	ptr = (char *)MALLOC(strlen(str) + 1);
 	if (ptr == NULL) {
-		log(LOG_NOTICE, "out of memory (newstr)");
+		logg(LOG_NOTICE, "out of memory (newstr)");
 		return NULL;
 	}
 	strcpy(ptr, str);
@@ -109,7 +110,7 @@ char *buf;
 	tv_sub(&ct, &pt, &dt);
 	d.ru_maxrss = c.ru_maxrss - p.ru_maxrss;
 	d.ru_nivcsw = c.ru_nivcsw - p.ru_nivcsw;
-	log(LOG_INFO, "time=%d.%03d+%d.%03d/%d.%03d rss=%ld nivcsw=%ld (%s)",
+	logg(LOG_INFO, "time=%d.%03d+%d.%03d/%d.%03d rss=%ld nivcsw=%ld (%s)",
 		d.ru_utime.tv_sec, d.ru_utime.tv_usec/1000,
 		d.ru_stime.tv_sec, d.ru_stime.tv_usec/1000,
 		dt.tv_sec, dt.tv_usec/1000,
@@ -139,7 +140,7 @@ char *name;
 	}
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "hash_func(%s) = %d", name, val);
+	logg(LOG_DEBUG, "hash_func(%s) = %d", name, val);
 
 	return val;
 }
@@ -165,7 +166,7 @@ struct dns_stat ***ptr;
                 if (strcasecmp(qp->name, name) == 0)
 		{
 			if (cnf.debug & DEBUG_HASH)
-			log(LOG_DEBUG, "query %s found in hash table", name);
+			logg(LOG_DEBUG, "query %s found in hash table", name);
                         return qp;
 		}
                 qp = qp->next;
@@ -194,7 +195,7 @@ struct domain ***ptr;
                 if (strcasecmp(domp->name, name) == 0)
 		{
 			if (cnf.debug & DEBUG_HASH)
-			log(LOG_DEBUG, "domain %s found in hash table", name);
+			logg(LOG_DEBUG, "domain %s found in hash table", name);
                         return domp;
 		}
                 domp = domp->hash;
@@ -211,7 +212,7 @@ struct domain *newdomp;
 	int idx = hash_func(name);
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "enter domain %s into hash table", name);
+	logg(LOG_DEBUG, "enter domain %s into hash table", name);
 
 	newdomp->hash = hash_domain[idx];
 	hash_domain[idx] = newdomp;
@@ -228,21 +229,21 @@ char *name;
 	int idx = hash_func(name);
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "look up domain(%s) index(%d)", name, idx);
+	logg(LOG_DEBUG, "look up domain(%s) index(%d)", name, idx);
 	dom_r_p = hash_domain_rcpts[idx];
 	while (dom_r_p != NULL)
 	{
 		if (strcasecmp(dom_r_p->name, name) == 0)
 		{
 			if (cnf.debug & DEBUG_HASH)
-			log(LOG_DEBUG, "hit domain(%s) rcpts(%d) in hash table",
+			logg(LOG_DEBUG, "hit domain(%s) rcpts(%d) in hash table",
 				name, dom_r_p->rcpts);
 			return dom_r_p->rcpts;
 		}
 		dom_r_p = dom_r_p->hash;
 	}
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "domain(%s) not found in hash table, return %d",
+	logg(LOG_DEBUG, "domain(%s) not found in hash table, return %d",
 		name, cnf.rcpts_trans);
 	return cnf.rcpts_trans;
 }
@@ -255,7 +256,7 @@ struct domain_rcpts *newdom_r_p;
 	int idx = hash_func(name);
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "enter domain(%s) index(%d) rcpts(%d) into hash table",
+	logg(LOG_DEBUG, "enter domain(%s) index(%d) rcpts(%d) into hash table",
 		name, idx, newdom_r_p->rcpts);
 
 	newdom_r_p->hash = hash_domain_rcpts[idx];
@@ -290,7 +291,7 @@ rcpts_def_load()
 	fp = fopen(cnf.rcpts_def, "r");
 	if (fp == NULL)
 	{
-		log(LOG_NOTICE, "file %s not found", cnf.rcpts_def);
+		logg(LOG_NOTICE, "file %s not found", cnf.rcpts_def);
 		return -1;
 	}
 	/* read cnf.rcpts_def */
@@ -337,7 +338,7 @@ rcpts_def_load()
 			dom_r_p = (struct domain_rcpts*) MALLOC(sizeof(struct domain_rcpts));
 			if (dom_r_p == NULL)
 			{
-				log(LOG_NOTICE, "out of memory (domain_rcpts)");
+				logg(LOG_NOTICE, "out of memory (domain_rcpts)");
 				fclose(fp);
 				return -1;
 			}
@@ -345,7 +346,7 @@ rcpts_def_load()
 			dom_r_p->name  = newstr(strp);
 			if(dom_r_p->name == NULL)
 			{
-				log(LOG_NOTICE, "out of memory (dom_r_p->name)");
+				logg(LOG_NOTICE, "out of memory (dom_r_p->name)");
 				fclose(fp);
 				return -1;
 			}
@@ -356,7 +357,7 @@ rcpts_def_load()
 	fclose(fp);
 	cnf.rcpts_trans = hash_domain_rcpts_lookup(DEFAULTRCPTS);
 	if (cnf.debug & DEBUG_TRANS)
-	log(LOG_DEBUG, "set cnf.rcpts_trans = %d", cnf.rcpts_trans);
+	logg(LOG_DEBUG, "set cnf.rcpts_trans = %d", cnf.rcpts_trans);
 	return 0;
 }
 
@@ -381,7 +382,7 @@ struct host ***ptr;
                 if (strcasecmp(hostp->name, name) == 0)
 		{
 			if (cnf.debug & DEBUG_HASH)
-			log(LOG_DEBUG, "host %s found in hash table", name);
+			logg(LOG_DEBUG, "host %s found in hash table", name);
                         return hostp;
 		}
                 hostp = hostp->hash;
@@ -398,7 +399,7 @@ struct host *newhostp;
 	int idx = hash_func(name);
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "enter host %s into hash table", name);
+	logg(LOG_DEBUG, "enter host %s into hash table", name);
 
 	newhostp->hash = hash_host[idx];
 	hash_host[idx] = newhostp;
@@ -426,7 +427,7 @@ struct host_map ***ptr;
                 if (strcasecmp(mapp->name, name) == 0)
 		{
 			if (cnf.debug & DEBUG_HASH)
-			log(LOG_DEBUG, "map %s found in hash table", name);
+			logg(LOG_DEBUG, "map %s found in hash table", name);
                         return mapp;
 		}
                 mapp = mapp->hash;
@@ -442,7 +443,7 @@ struct host_map *newmapp;
 	int idx = hash_func(name);
 
 	if (cnf.debug & DEBUG_HASH)
-	log(LOG_DEBUG, "enter map %s into hash table", name);
+	logg(LOG_DEBUG, "enter map %s into hash table", name);
 
 	newmapp->hash = hash_map[idx];
 	hash_map[idx] = newmapp;
@@ -462,7 +463,7 @@ host_map_load()
 	fp = fopen(cnf.map, "r");
 	if (fp == NULL)
 	{
-		log(LOG_NOTICE, "map %s not found", cnf.map);
+		logg(LOG_NOTICE, "map %s not found", cnf.map);
 		return -1;	/* should be TEMPFAIL */
 	}
 	while (fgets(buf, sizeof(buf), fp) != NULL)
@@ -580,7 +581,7 @@ host_map_load()
 				}
 				argp++;
 			}
-			log(LOG_DEBUG, "reading map: domain = %s, map = %s",
+			logg(LOG_DEBUG, "reading map: domain = %s, map = %s",
 				mapp->name, tmpbuf);
 		}
 	}
@@ -604,7 +605,7 @@ char *domain;
 	while (p != NULL)
 	{
 		if (cnf.debug & DEBUG_MAP)
-		log(LOG_DEBUG, "searching host map for %s: %s", domain, p);
+		logg(LOG_DEBUG, "searching host map for %s: %s", domain, p);
 
 		if ((mapp = hash_map_lookup(p, NULL)) != NULL)
 		{
